
#version 130              // delete this line if using GLSL 1.2
precision mediump float;  // delete this line if using GLSL 1.2


//// uniform shader-parameters
uniform vec3 light_position;
uniform vec4 light_color;
uniform vec4 ambient_color;
uniform sampler2D colorTexture;

// fragment-shader input variables
in vec2 frag_texCoord;
in vec3 world_normal;
in vec4 world_position;

// fragment-shader output variable (-> stored in the frame-buffer, i.e. "the pixel you see")
out vec4 fragColor;


void main()
{
	// renormalize and homogenize input variables
	vec3 normal = normalize(world_normal);
    vec3 position = world_position.xyz / world_position.w;
    
    // calculate the light-direction
    vec3 light_dir = normalize(light_position - position);
    
    //get texel color
    vec4 texel = texture2D(colorTexture, frag_texCoord);

    // calculate lighting
    vec4 ambient = ambient_color * texel;
    vec4 diffuse = texel * light_color * max(0.0, dot(normal, light_dir));
    
    // write color to output
    fragColor = ambient + diffuse;
}