#version 130              // delete this line if using GLSL 1.2
precision mediump float;  // delete this line if using GLSL 1.2


// uniform shader-parameters
uniform float weightingFact;
uniform mat4 perspective;
uniform mat4 view;
uniform mat4 model;
uniform vec4 clipPlane;

// vertex-shader input variables
in vec3 normal;
in vec4 vertex;
in vec3 normalB;
in vec4 vertexB;
in vec2 texCoord;

// vertex-shader output variables (passed to fragment-shader)
out vec2 frag_texCoord;
out vec3 world_normal;
out vec4 world_position;
out float gl_ClipDistance[1];


void main()
{
	vec4 blendedPos_vert = mix(vertex, vertexB, weightingFact);
	vec3 blendedPos_norm = mix(normal, normalB, weightingFact);
	vec3 normal = normalize(blendedPos_norm);
		
	// transform normal into world space
	vec4 normal4 = vec4(normal, 0.0f);
	world_normal = (model * normal4).xyz;
		
	// transform vertex to world-space
	world_position = model * blendedPos_vert;
	
	gl_ClipDistance[0] = dot(world_position, clipPlane);
		
	// just pass color to fragment-shader
	frag_texCoord = texCoord;

	// transform vertex down the pipeline
	gl_Position = perspective * view * model * blendedPos_vert;
}
